@@ -6,7 +6,7 @@ const _ = require('lodash');
 require('../util/fips');
 
 const assert = require('assert');
-const Ajv = require('ajv');
+const { default: Ajv } = require('ajv');
 const crypto = require('crypto');
 const util = require('util');
 const EventEmitter = require('events').EventEmitter;
@@ -16,6 +16,7 @@ const P = require('./promise');
 const dbg = require('./debug_module')(__filename);
 const common_api = require('../api/common_api');
 const schema_utils = require('./schema_utils');
+const schema_keywords = require('./schema_keywords');
 const mongodb = require('mongodb');
 const mongo_to_pg = require('mongo-query-to-postgres-jsonb');
 const fs = require('fs');
@@ -1302,12 +1303,14 @@ class PostgresClient extends EventEmitter {
         };
 
         PostgresClient.implements_interface(this);
-        this._ajv = new Ajv({ verbose: true, schemaId: 'auto', allErrors: true });
-        this._ajv.addKeyword('date', schema_utils.KEYWORDS.date);
-        this._ajv.addKeyword('idate', schema_utils.KEYWORDS.idate);
-        this._ajv.addKeyword('objectid', schema_utils.KEYWORDS.objectid);
-        this._ajv.addKeyword('binary', schema_utils.KEYWORDS.binary);
-        this._ajv.addKeyword('wrapper', schema_utils.KEYWORDS.wrapper);
+        this._ajv = new Ajv({ verbose: true, allErrors: true });
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.methods);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.doc);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.date);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.idate);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.objectid);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.binary);
+        this._ajv.addKeyword(schema_keywords.KEYWORDS.wrapper);
         this._ajv.addSchema(common_api);
 
         // this.emit('reconnect');
